local _, ns = ...
local E, C, M, L, P, oUF = ns.E, ns.C, ns.M, ns.L, ns.P, ns.oUF
local UF = P:AddModule("UnitFrames")

-- Lua
local _G = getfenv(0)
local string = _G.string
local getmetatable = _G.getmetatable
local pairs = _G.pairs
local tonumber = _G.tonumber
local unpack = _G.unpack

-- Mine
local isInit = false
local objects = {}

local function LSUnitFrame_OnEnter(self)
	if self.__owner then
		self = self.__owner
	end

	_G.UnitFrame_OnEnter(self)

	if string.match(self:GetName(), "LSPetFrame") then
		_G.PartyMemberBuffTooltip:ClearAllPoints()
		_G.PartyMemberBuffTooltip:SetPoint("BOTTOMRIGHT", self, "TOPLEFT", 4, -4)
		_G.PartyMemberBuffTooltip_Update(self)
	end
end

local function LSUnitFrame_OnLeave(self)
	if self.__owner then
		self = self.__owner
	end

	_G.UnitFrame_OnLeave(self)

	if string.match(self:GetName(), "LSPetFrame") then
		_G.PartyMemberBuffTooltip:Hide()
	end
end

local function UnitFrameConstructor(frame, unit)
	frame:RegisterForClicks("AnyUp")
	frame:SetScript("OnEnter", LSUnitFrame_OnEnter)
	frame:SetScript("OnLeave", LSUnitFrame_OnLeave)

	if unit == "player" then
		UF:ConstructPlayerFrame(frame)
	elseif unit == "pet" then
		UF:ConstructPetFrame(frame)
	elseif unit == "target" then
		UF:ConstructTargetFrame(frame)
	elseif unit == "targettarget" then
		UF:ConstructTargetTargetFrame(frame)
	elseif unit == "focus" then
		UF:ConstructFocusFrame(frame)
	elseif unit == "focustarget" then
		UF:ConstructFocusTargetFrame(frame)
	elseif unit == "boss1" then
		UF:ConstructBossFrame(frame)
	elseif unit == "boss2" then
		UF:ConstructBossFrame(frame)
	elseif unit == "boss3" then
		UF:ConstructBossFrame(frame)
	elseif unit == "boss4" then
		UF:ConstructBossFrame(frame)
	elseif unit == "boss5" then
		UF:ConstructBossFrame(frame)
	end
end

local function MainConstructor()
	oUF:RegisterStyle("LSv2", UnitFrameConstructor)
	oUF:SetActiveStyle("LSv2")

	if C.db.profile.units[C.db.char.layout].player.enabled then
		objects["player"] = oUF:Spawn("player", "LSPlayerFrame")
		objects["pet"] = oUF:Spawn("pet", "LSPetFrame")

		UF:UpdatePlayerFrame(objects["player"])
		UF:UpdatePetFrame(objects["pet"])

		if not C.db.profile.units[C.db.char.layout].player.castbar.enabled then
			UF:EnableDefaultCastingBars()
		end
	end

	if C.db.profile.units[C.db.char.layout].target.enabled then
		objects["target"] = oUF:Spawn("target", "LSTargetFrame")
		objects["targettarget"] = oUF:Spawn("targettarget", "LSTargetTargetFrame")

		UF:UpdateTargetFrame(objects["target"])
		UF:UpdateTargetTargetFrame(objects["targettarget"])
	end

	if C.db.profile.units[C.db.char.layout].focus.enabled then
		objects["focus"] = oUF:Spawn("focus", "LSFocusFrame")
		objects["focustarget"] = oUF:Spawn("focustarget", "LSFocusTargetFrame")

		UF:UpdateFocusFrame(objects["focus"])
		UF:UpdateFocusTargetFrame(objects["focustarget"])
	end

	if C.db.profile.units[C.db.char.layout].boss.enabled then
		for i = 1, 5 do
			objects["boss"..i] = oUF:Spawn("boss"..i, "LSBoss"..i.."Frame")

			UF:UpdateBossFrame(objects["boss"..i])
		end
	end

	for unit, object in pairs(objects) do
		if string.match(unit, "^boss") then
			local id = tonumber(string.match(unit, "boss(%d)"))

			if id == 1 then
				UF:CreateBossHolder()

				object:SetPoint("TOPRIGHT", "LSBossHolder", "TOPRIGHT", 0, 0)
			else
				object:SetPoint("TOP", "LSBoss"..(id - 1).."Frame", "BOTTOM", 0, -28)
			end
		else
			object:SetPoint(unpack(C.db.profile.units[C.db.char.layout][unit].point))
			E:CreateMover(object)
		end
	end
end

--------------
-- SETTINGS --
--------------

function UF:SpawnFrame(unit)
	if unit == "player" and not objects["player"] then
		objects["player"] = oUF:Spawn("player", "LSPlayerFrame")
		objects["pet"] = oUF:Spawn("pet", "LSPetFrame")

		objects["player"]:SetPoint(unpack(C.db.profile.units[C.db.char.layout].player.point))
		E:CreateMover(objects["player"])

		objects["pet"]:SetPoint(unpack(C.db.profile.units[C.db.char.layout].pet.point))
		E:CreateMover(objects["pet"])

		UF:UpdatePlayerFrame(objects["player"])
		UF:UpdatePetFrame(objects["pet"])

		if not C.db.profile.units[C.db.char.layout].player.castbar.enabled then
			UF:EnableDefaultCastingBars()
		end

		return true
	elseif unit == "target" and not objects["target"] then
		objects["target"] = oUF:Spawn("target", "LSTargetFrame")
		objects["targettarget"] = oUF:Spawn("targettarget", "LSTargetTargetFrame")

		objects["target"]:SetPoint(unpack(C.db.profile.units[C.db.char.layout].target.point))
		E:CreateMover(objects["target"])

		objects["targettarget"]:SetPoint(unpack(C.db.profile.units[C.db.char.layout].targettarget.point))
		E:CreateMover(objects["targettarget"])

		UF:UpdateTargetFrame(objects["target"])
		UF:UpdateTargetTargetFrame(objects["targettarget"])

		return true
	elseif unit == "focus" and not objects["focus"] then
		objects["focus"] = oUF:Spawn("focus", "LSFocusFrame")
		objects["focustarget"] = oUF:Spawn("focustarget", "LSFocusTargetFrame")

		objects["focus"]:SetPoint(unpack(C.db.profile.units[C.db.char.layout].focus.point))
		E:CreateMover(objects["focus"])

		objects["focustarget"]:SetPoint(unpack(C.db.profile.units[C.db.char.layout].focustarget.point))
		E:CreateMover(objects["focustarget"])

		UF:UpdateFocusFrame(objects["focus"])
		UF:UpdateFocusTargetFrame(objects["focustarget"])

		return true
	elseif unit == "boss" and not objects["boss1"] then
		UF:CreateBossHolder()

		for i = 1, 5 do
			objects["boss"..i] = oUF:Spawn("boss"..i, "LSBoss"..i.."Frame")

			UF:UpdateBossFrame(objects["boss"..i])

			if i == 1 then
				objects["boss"..i]:SetPoint("TOPRIGHT", "LSBossHolder", "TOPRIGHT", 0, 0)
			else
				objects["boss"..i]:SetPoint("TOP", objects["boss"..(i - 1)], "BOTTOM", 0, -28)
			end
		end

		return true
	end
end

function UF:EnableElement(unit, element)
	if objects[unit] then
		objects[unit]:EnableElement(element)
		objects[unit]:UpdateAllElements("EnableElement")
	end
end

function UF:DisableElement(unit, element)
	if objects[unit] then
		objects[unit]:DisableElement(element)
	end
end

function UF:UpdateUnitFrames()
	for _, frame in pairs(objects) do
		frame:UpdateAllElements("ForceUpdate")
	end
end

function UF:EnableDefaultCastingBars()
	_G.CastingBarFrame.Show = getmetatable(_G.CastingBarFrame).__index.Show
	_G.CastingBarFrame.unit = nil
	_G.CastingBarFrame_OnLoad(_G.CastingBarFrame, "player", true, false)

	_G.PetCastingBarFrame.Show = getmetatable(_G.PetCastingBarFrame).__index.Show
	_G.PetCastingBarFrame.unit = nil
	_G.PetCastingBarFrame_OnLoad(_G.PetCastingBarFrame)
end

-----------------
-- INITIALISER --
-----------------

function UF:IsInit()
	return isInit
end

function UF:Init()
	if not isInit and C.db.char.units.enabled then
		oUF:Factory(MainConstructor)

		if not C.db.profile.units[C.db.char.layout].player.castbar then
			if objects["player"] then
				objects["player"]:DisableElement("Castbar")
			end

			if objects["pet"] then
				objects["pet"]:DisableElement("Castbar")
			end

			UF:EnableDefaultCastingBars()
		end

		if not C.db.profile.units[C.db.char.layout].boss.castbar then
			if objects["boss1"] then
				objects["boss1"]:DisableElement("Castbar")
			end

			if objects["boss2"] then
				objects["boss2"]:DisableElement("Castbar")
			end

			if objects["boss3"] then
				objects["boss3"]:DisableElement("Castbar")
			end

			if objects["boss4"] then
				objects["boss4"]:DisableElement("Castbar")
			end

			if objects["boss5"] then
				objects["boss5"]:DisableElement("Castbar")
			end
		end

		if objects["target"] then
			if not C.db.profile.units[C.db.char.layout].target.castbar then
				objects["target"]:DisableElement("Castbar")
			end

			if not C.db.profile.units[C.db.char.layout].target.auras.enabled then
				objects["target"]:DisableElement("Aura")
			end
		end

		if objects["focus"] then
			if not C.db.profile.units[C.db.char.layout].focus.castbar then
				objects["focus"]:DisableElement("Castbar")
			end

			if not C.db.profile.units[C.db.char.layout].focus.auras.enabled then
				objects["focus"]:DisableElement("Aura")
			end
		end

		-- Finalise
		isInit = true

		return true
	end
end
